import 'package:hive/hive.dart';
import 'package:osam/src/domain/persist.dart';
import 'package:path_provider/path_provider.dart';

class PersistRepository<ST, US> implements Persist<ST, US> {
  Box<ST> _persist;
  Box<US> _uiPersist;

  @override
  Future<void> initPersist(String appName) async {
    Hive.init((await getApplicationDocumentsDirectory()).path);
    _persist ??= await Hive.openBox(appName);
    _uiPersist ??= await Hive.openBox(appName + '_ui_');
  }

  @override
  Future<void> save(ST state, US uiState) async {
    await _persist.put('appState', state);
    if (uiState != null) await _uiPersist.put('uiState', uiState);
  }

  @override
  ST getState() => _persist.get('appState');

  @override
  US getUiState() => _uiPersist.get('uiState');

  @override
  Future<void> delete() async {
    await _persist.delete('appState');
    await _uiPersist.delete('uiState');
  }

  @override
  Future<void> deleteFromDisk(String appName) async {
    await Hive.deleteBoxFromDisk(appName);
    await Hive.deleteBoxFromDisk(appName + '_ui_');
  }

  @override
  Future<void> deleteHive() async => await Hive.deleteFromDisk();

  @override
  bool get isPersistEnabled => _persist == null ? false : true;
}
